@@ -4,18 +4,19 @@ The "base" environment is one that makes use of the CLI directly and is not nece
 integration (CI) environment. Common functionality is provided where possible and CI specific features are
 designated as abstract methods to be defined in specific CI environments.
 """
-import json
 import os
+import shlex
 import shutil
 import subprocess
-import tempfile
 import textwrap
 import urllib.parse
 from abc import ABC, abstractmethod
 from argparse import Namespace
+from collections import OrderedDict
 from pathlib import Path
 from typing import List, Optional
 
+import pathspec
 from backports.cached_property import cached_property
 from connect.utils.terminal.markdown import render
 from packaging.version import Version
@@ -29,31 +30,13 @@ from phylum.ci.constants import (
     PROJECT_THRESHOLD_OPTIONS,
     SUCCESS_COMMENT,
 )
+from phylum.ci.git import git_hash_object, git_repo_name
+from phylum.ci.lockfile import Lockfile, Lockfiles
 from phylum.constants import MIN_CLI_VER_INSTALLED, SUPPORTED_LOCKFILES, TOKEN_ENVVAR_NAME
 from phylum.init.cli import get_phylum_bin_path
 from phylum.init.cli import main as phylum_init
 
 
-def detect_lockfile() -> Optional[Path]:
-    """Detect the lockfile in use and return it.
-
-    This method has some assumptions:
-      * It is called from the root of a repository
-      * There is only one lockfile allowed
-    """
-    cwd = Path.cwd()
-    lockfiles_in_cwd = [path.resolve() for lockfile_glob in SUPPORTED_LOCKFILES for path in cwd.glob(lockfile_glob)]
-    lockfiles_in_cwd = [path for path in lockfiles_in_cwd if path.stat().st_size]
-    if not lockfiles_in_cwd:
-        return None
-    if len(lockfiles_in_cwd) > 1:
-        lockfiles = ", ".join(str(lockfile) for lockfile in lockfiles_in_cwd)
-        print(f" [!] Multiple lockfiles detected: {lockfiles}")
-        raise SystemExit(" [!] Only one lockfile is supported at this time. Consider specifying it with `--lockfile`.")
-    lockfile = lockfiles_in_cwd[0]
-    return lockfile
-
-
 class CIBase(ABC):
     """Provide methods for a basic CI environment."""
 
@@ -65,35 +48,24 @@ class CIBase(ABC):
           * call `super().__init__(args)`
           * define `self.ci_platform_name`
         """
-        self.args = args
-        self._phylum_project_file = Path.cwd().joinpath(".phylum_project").resolve()
-
-        # The lockfile specified as a script argument will be used, if provided.
-        # Otherwise, an attempt will be made to automatically detect the lockfile.
-        provided_lockfile: Path = args.lockfile
-        if provided_lockfile and provided_lockfile.exists() and provided_lockfile.stat().st_size:
-            self._lockfile = provided_lockfile.resolve()
-            print(f" [-] Provided lockfile: {self._lockfile}")
-        else:
-            detected_lockfile = detect_lockfile()
-            if detected_lockfile:
-                self._lockfile = detected_lockfile
-                print(f" [-] Detected lockfile: {self._lockfile}")
-            else:
-                raise SystemExit(
-                    " [!] A lockfile is required and was not detected. Consider specifying one with `--lockfile`."
-                )
-
+        self._args = args
         self._all_deps = args.all_deps
         self._force_analysis = args.force_analysis
-        self._phylum_project = args.project
-        self._phylum_group = args.group
+
+        # Create a copy of the original `.phylum_project` file values, when the file exists.
+        # This is necessary because it is possible that user-provided values for the project and
+        # group are given, which causes the file to be overwritten when creating that project.
+        self._phylum_project_file = Path.cwd().joinpath(".phylum_project").resolve()
+        self._project_file_already_existed = self._phylum_project_file.exists()
+        self._project_settings = {}
+        if self._project_file_already_existed:
+            yaml = YAML()
+            self._project_settings = yaml.load(self._phylum_project_file.read_text(encoding="utf-8"))
 
         # Ensure all pre-requisites are met and bail at the earliest opportunity when they aren't
         self._check_prerequisites()
         print(" [+] All pre-requisites met")
 
-        self._cli_path: Optional[Path] = None
         self.gbl_failed = False
         self.gbl_incomplete = False
         self.incomplete_pkgs: Packages = []
@@ -106,19 +78,73 @@ class CIBase(ABC):
         if args.token is not None:
             token = args.token
             os.environ[TOKEN_ENVVAR_NAME] = args.token
-        self.args.token = token
+        self._args.token = token
 
-        self._lockfile_changed = self._is_lockfile_changed(self.lockfile)
+        self.ensure_project_exists()
 
     @property
-    def lockfile(self) -> Path:
-        """Get the package lockfile."""
-        return self._lockfile
+    def args(self) -> Namespace:
+        """Get the namespace arguments provided on the command line."""
+        return self._args
 
-    @property
-    def is_lockfile_changed(self) -> bool:
-        """Get the lockfile's modification status."""
-        return self._lockfile_changed
+    @cached_property
+    def lockfiles(self) -> Lockfiles:
+        """Get the package lockfile(s) in lexicographic order.
+
+        The package lockfile(s) can be specified as an option or contained in the `.phylum_project` file.
+        Lockfiles provided as an input option will be preferred over any entries in the `.phylum_project` file.
+
+        When no valid lockfiles are provided otherwise, an attempt will be made to automatically detect them.
+        """
+        arg_lockfiles: Optional[List[List[Path]]] = self.args.lockfile
+        if arg_lockfiles:
+            # flatten the list of lists
+            provided_arg_lockfiles = [path for sub_list in arg_lockfiles for path in sub_list]
+            print(f" [+] Lockfile(s) provided as arguments: {provided_arg_lockfiles}")
+            valid_lockfiles = self.filter_lockfiles(provided_arg_lockfiles)
+            if valid_lockfiles:
+                print(f" [-] Valid provided lockfiles: {valid_lockfiles}")
+                return valid_lockfiles
+
+        print(" [+] No valid lockfiles were provided as arguments. Checking the `.phylum_project` file ...")
+        lockfile_entries: List[OrderedDict] = self._project_settings.get("lockfiles", [])
+        lockfile_paths = [lockfile_entry.get("path") for lockfile_entry in lockfile_entries]
+        provided_project_lockfiles = [Path(lockfile) for lockfile in lockfile_paths if lockfile]
+        if provided_project_lockfiles:
+            print(f" [+] Lockfile(s) provided in `.phylum_project` file: {provided_project_lockfiles}")
+            valid_lockfiles = self.filter_lockfiles(provided_project_lockfiles)
+            if valid_lockfiles:
+                print(f" [-] Valid provided lockfiles: {valid_lockfiles}")
+                return valid_lockfiles
+
+        print(" [+] No valid lockfiles in the `.phylum_project` file. An attempt will be made to detect lockfiles.")
+        detected_lockfiles = detect_lockfiles()
+        if detected_lockfiles:
+            print(f" [+] Detected lockfile(s): {detected_lockfiles}")
+            valid_lockfiles = self.filter_lockfiles(detected_lockfiles)
+            if valid_lockfiles:
+                print(f" [-] Valid detected lockfiles: {valid_lockfiles}")
+                return valid_lockfiles
+
+        raise SystemExit(" [!] No valid lockfiles were detected. Consider specifying at least one with `--lockfile`.")
+
+    def filter_lockfiles(self, provided_lockfiles: List[Path]) -> Lockfiles:
+        """Filter potential lockfiles and return the valid ones in sorted order."""
+        lockfiles = []
+        for provided_lockfile in provided_lockfiles:
+            if not provided_lockfile.exists():
+                raise SystemExit(f" [!] Provided lockfile does not exist: {provided_lockfile}")
+            if not provided_lockfile.stat().st_size:
+                print(f" [!] Provided lockfile is an empty file: {provided_lockfile}")
+                continue
+            cmd = [str(self.cli_path), "parse", str(provided_lockfile)]
+            # stdout and stderr are piped to DEVNULL because we only care about the return code.
+            # pylint: disable-next=subprocess-run-check ; we want return code here and don't want to raise when non-zero
+            if bool(subprocess.run(cmd, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL).returncode):
+                print(f" [!] Provided lockfile failed to parse as a known lockfile type: {provided_lockfile}")
+                continue
+            lockfiles.append(Lockfile(provided_lockfile, self.cli_path, self.common_ancestor_commit))
+        return sorted(lockfiles)
 
     @property
     def all_deps(self) -> bool:
@@ -130,23 +156,32 @@ class CIBase(ABC):
         """Get the status of forcing an analysis."""
         return self._force_analysis
 
-    @property
+    @cached_property
     def phylum_project(self) -> str:
         """Get the effective Phylum project name in use.
 
         The Phylum project name can be specified as an option or contained in the `.phylum_project` file.
         A project name provided as an input option will be preferred over an entry in the `.phylum_project` file.
 
-        Return `None` when the project name is not available.
+        When no project name is provided through options or project file, a project name will be provided by detecting
+        the git repository name. The goal is a unique and deterministic project name for each git repository submitted
+        by the same Phylum user account.
         """
-        # Project supplied as an option
-        if self._phylum_project:
-            return self._phylum_project
+        project_name = self.args.project
+        if project_name:
+            print(f" [+] Project name provided as argument: {project_name}")
+            return project_name
 
-        # Project supplied in `.phylum_project`
-        yaml = YAML()
-        settings_dict = yaml.load(self.phylum_project_file.read_text(encoding="utf-8"))
-        return settings_dict.get("name")
+        print(" [+] Project name not provided as argument. Checking the `.phylum_project` file ...")
+        project_name = self._project_settings.get("name")
+        if project_name:
+            print(f" [+] Project name provided in `.phylum_project` file: {project_name}")
+            return project_name
+
+        print(" [+] Project name not found in the `.phylum_project` file or file does not exist. Detecting instead ...")
+        project_name = git_repo_name()
+        print(f" [+] Project name detected from git repository name: {project_name}")
+        return project_name
 
     @property
     def phylum_group(self) -> Optional[str]:
@@ -158,218 +193,23 @@ class CIBase(ABC):
         Return `None` when the group name is not available.
         """
         # Group supplied on command-line
-        if self._phylum_project:
-            return self._phylum_group
+        # A group can not be specified without a project
+        if self.args.group and self.args.project:
+            return self.args.group
 
         # Group supplied in `.phylum_project`
-        yaml = YAML()
-        settings_dict = yaml.load(self.phylum_project_file.read_text(encoding="utf-8"))
-        return settings_dict.get("group_name")
+        # Don't "mix" a project supplied as an option with a group that wasn't
+        if not self.args.project:
+            return self._project_settings.get("group_name")
 
-    @property
-    def phylum_project_file(self) -> Path:
-        """Get the path to the Phylum project file `.phylum_project`."""
-        return self._phylum_project_file
-
-    @property
-    def cli_path(self) -> Optional[Path]:
-        """Get the path to the Phylum CLI binary."""
-        return self._cli_path
-
-    @property
-    def project_id(self) -> str:
-        """Get the GUID for the project."""
-        return self._project_id
+        return None
 
     @cached_property
-    def project_url(self) -> str:
-        """Construct a project URL and return it.
+    def cli_path(self) -> Path:
+        """Get the path to the Phylum CLI binary.
 
-        The project URL is used to access a particular project in the web UI, by label, and optionally with a group.
+        Check for an existing Phylum CLI and install it if needed.
         """
-        query = {"label": self.phylum_label}
-        if self.phylum_group is not None:
-            query["group"] = self.phylum_group
-        query_params = urllib.parse.urlencode(query, safe="/", quote_via=urllib.parse.quote)
-        project_url = f"https://app.phylum.io/projects/{self.project_id}?{query_params}"
-        return project_url
-
-    @property
-    def project_url_md(self) -> str:
-        """Construct a markdown link for viewing the project in the Phylum UI."""
-        return f"\n[View this project in the Phylum UI]({self.project_url})"
-
-    @property
-    def analysis_output(self) -> str:
-        """Get the output from the overall analysis, in markdown format."""
-        return self._analysis_output
-
-    @property
-    @abstractmethod
-    def phylum_label(self) -> str:
-        """Get a custom label for use when submitting jobs with `phylum analyze`.
-
-        Each CI platform/environment has unique ways of referencing events, PRs, branches, etc.
-        """
-        raise NotImplementedError()
-
-    @cached_property
-    @abstractmethod
-    def common_lockfile_ancestor_commit(self) -> Optional[str]:
-        """Find the common lockfile ancestor commit.
-
-        When found, it should be returned as a string of the SHA1 sum representing the commit.
-        When it can't be found (or there is an error), `None` should be returned.
-        """
-        raise NotImplementedError()
-
-    @abstractmethod
-    def _is_lockfile_changed(self, lockfile: Path) -> bool:
-        """Predicate for detecting if the given lockfile has changed."""
-        raise NotImplementedError()
-
-    @abstractmethod
-    def _check_prerequisites(self) -> None:
-        """Ensure the necessary pre-requisites are met and bail when they aren't.
-
-        The current pre-requisites for *all* CI environments/platforms are:
-          * A `.phylum_project` file exists at the working directory or project name is specified as an option
-          * A Phylum CLI version with the `parse` command
-          * Have `git` installed and available for use on the PATH
-        """
-        print(" [+] Confirming pre-requisites ...")
-
-        if self.phylum_project_file.exists():
-            print(f" [+] Existing `.phylum_project` file found at: {self.phylum_project_file}")
-        else:
-            print(f" [+] No existing `.phylum_project` file found at: {self.phylum_project_file}")
-            if self.phylum_project:
-                print(f" [+] A Phylum project will be used (or created) with the provided name: {self.phylum_project}")
-            else:
-                msg = """\
-                    [!] No Phylum project could be determined!
-                    [!] Consider specifying one with `--project` and optionally with the `--group` option."""
-                raise SystemExit(textwrap.dedent(msg))
-
-        if Version(self.args.version) < Version(MIN_CLI_VER_INSTALLED):
-            raise SystemExit(f" [!] The CLI version must be at least {MIN_CLI_VER_INSTALLED}")
-
-        if shutil.which("git"):
-            print(" [+] `git` binary found on the PATH")
-        else:
-            raise SystemExit(" [!] `git` is required to be installed and available on the PATH")
-
-    def post_output(self) -> None:
-        """Post the output of the analysis as markdown rendered for output to the terminal/logs.
-
-        Each implementation that offers analysis output in the form of comments on a pull/merge request should
-        ensure those comments are unique and not added multiple times as the review changes but the lockfile doesn't.
-        """
-        # Pull out the project URL link, which doesn't render well
-        output = self.analysis_output.replace(self.project_url_md, "")
-
-        # Post the markdown output, rendered for terminal/log output
-        print(f" [+] Analysis output:\n{render(output)}")
-
-        # Post the project URL link separately
-        print(f"View this project in the Phylum UI: {self.project_url}\n")
-
-    @cached_property
-    def current_lockfile_packages(self) -> Packages:
-        """Get the current lockfile packages.
-
-        This property expects the Phylum CLI path to be known and will raise `SystemExit` when that path is unknown.
-        """
-        if not self.cli_path:
-            raise SystemExit(" [!] Phylum CLI path is unknown. Try using the `init_cli` method first.")
-        try:
-            cmd = [str(self.cli_path), "parse", str(self.lockfile)]
-            parse_result = subprocess.run(cmd, check=True, capture_output=True, text=True).stdout.strip()
-        except subprocess.CalledProcessError as err:
-            print(f" [!] There was an error running the command: {' '.join(err.cmd)}")
-            print(f" [!] stdout:\n{err.stdout}")
-            print(f" [!] stderr:\n{err.stderr}")
-            raise SystemExit(f" [!] Is {self.lockfile} valid? If so, please report this as a bug.") from err
-        parsed_pkgs = json.loads(parse_result)
-        curr_lockfile_packages = [PackageDescriptor(**pkg) for pkg in parsed_pkgs]
-        return curr_lockfile_packages
-
-    @property
-    def previous_lockfile_object(self) -> Optional[str]:
-        """Get the previous git object for the lockfile."""
-        if not self.common_lockfile_ancestor_commit:
-            return None
-        try:
-            cmd = [
-                "git",
-                "rev-parse",
-                "--verify",
-                f"{self.common_lockfile_ancestor_commit}:{self.lockfile.name}",
-            ]
-            prev_lockfile_object = subprocess.run(cmd, check=True, capture_output=True, text=True).stdout.strip()
-        except subprocess.CalledProcessError as err:
-            # There could be a true error, but the working assumption when here is a previous version does not exist
-            print(f" [?] There *may* be an issue with the attempt to get the previous lockfile object: {err}")
-            print(f" [?] stdout:\n{err.stdout}")
-            print(f" [?] stderr:\n{err.stderr}")
-            print(" [+] Assuming a previous lockfile version does not exist ...")
-            prev_lockfile_object = None
-        return prev_lockfile_object
-
-    def get_previous_lockfile_packages(self, prev_lockfile_object: str) -> Packages:
-        """Get the previous lockfile packages from the corresponding git object and return them.
-
-        Expects the Phylum CLI path to be known and will raise `SystemExit` when that path is unknown.
-        """
-        if not self.cli_path:
-            raise SystemExit(" [!] Phylum CLI path is unknown. Try using the `init_cli` method first.")
-
-        with tempfile.NamedTemporaryFile(mode="w+") as prev_lockfile_fd:
-            try:
-                cmd = ["git", "cat-file", "blob", prev_lockfile_object]
-                prev_lockfile_contents = subprocess.run(cmd, check=True, capture_output=True, text=True).stdout
-                prev_lockfile_fd.write(prev_lockfile_contents)
-                prev_lockfile_fd.flush()
-            except subprocess.CalledProcessError as err:
-                print(f" [!] There was an error running the command: {' '.join(err.cmd)}")
-                print(f" [!] stdout:\n{err.stdout}")
-                print(f" [!] stderr:\n{err.stderr}")
-                print(" [!] Due to error, assuming no previous lockfile packages. Please report this as a bug.")
-                return []
-            try:
-                cmd = [str(self.cli_path), "parse", prev_lockfile_fd.name]
-                parse_result = subprocess.run(cmd, check=True, capture_output=True, text=True).stdout.strip()
-            except subprocess.CalledProcessError as err:
-                print(f" [!] There was an error running the command: {' '.join(err.cmd)}")
-                print(f" [!] stdout:\n{err.stdout}")
-                print(f" [!] stderr:\n{err.stderr}")
-                print(" [!] Due to error, assuming no previous lockfile packages. Please report this as a bug.")
-                return []
-
-        parsed_pkgs = json.loads(parse_result)
-        prev_lockfile_packages = [PackageDescriptor(**pkg) for pkg in parsed_pkgs]
-        return prev_lockfile_packages
-
-    def get_new_deps(self) -> Packages:
-        """Get the new dependencies added to the lockfile and return them in sorted order."""
-        curr_lockfile_packages = self.current_lockfile_packages
-
-        prev_lockfile_object = self.previous_lockfile_object
-        if not prev_lockfile_object:
-            print(" [+] No previous lockfile object found. Assuming all packages in the current lockfile are new.")
-            return curr_lockfile_packages
-
-        prev_lockfile_packages = self.get_previous_lockfile_packages(prev_lockfile_object)
-
-        prev_pkg_set = set(prev_lockfile_packages)
-        curr_pkg_set = set(curr_lockfile_packages)
-        new_deps = sorted(curr_pkg_set.difference(prev_pkg_set))
-        print(f" [+] New dependencies: {new_deps}")
-
-        return new_deps
-
-    def init_cli(self) -> None:
-        """Check for an existing Phylum CLI install, install it if needed, and set the path class instance variable."""
         specified_version = self.args.version
         # fmt: off
         install_args = [
@@ -396,9 +236,8 @@ class CIBase(ABC):
                     print(" [+] Version checks succeeded. Using existing version.")
 
         cli_path, cli_version = get_phylum_bin_path()
-        print(f" [+] Using Phylum CLI instance: {cli_version} at {str(cli_path)}")
-
-        self._cli_path = cli_path
+        if cli_path is None:
+            raise SystemExit(" [!] Failed to initialize the Phylum CLI")
 
         # Exit condition: a Phylum API key should be in place or available at this point.
         # Ensure stdout is piped to DEVNULL, to keep the token from being printed in (CI log) output.
@@ -407,8 +246,172 @@ class CIBase(ABC):
         if bool(subprocess.run(cmd, stdout=subprocess.DEVNULL).returncode):
             raise SystemExit(" [!] A Phylum API key is required to continue.")
 
+        print(f" [+] Using Phylum CLI instance: {cli_version} at {str(cli_path)}")
+        return cli_path
+
+    @property
+    def project_id(self) -> str:
+        """Get the GUID for the project."""
+        return self._project_id
+
+    @cached_property
+    def project_url(self) -> str:
+        """Construct a project URL and return it.
+
+        The project URL is used to access a particular project in the web UI, by label, and optionally with a group.
+        """
+        query = {"label": self.phylum_label}
+        if self.phylum_group:
+            query["group"] = self.phylum_group
+        query_params = urllib.parse.urlencode(query, safe="/", quote_via=urllib.parse.quote)
+        project_url = f"https://app.phylum.io/projects/{self.project_id}?{query_params}"
+        return project_url
+
+    @property
+    def project_url_md(self) -> str:
+        """Construct a markdown link for viewing the project in the Phylum UI."""
+        return f"\n[View this project in the Phylum UI]({self.project_url})"
+
+    @property
+    def analysis_output(self) -> str:
+        """Get the output from the overall analysis, in markdown format."""
+        return self._analysis_output
+
+    @property
+    @abstractmethod
+    def phylum_label(self) -> str:
+        """Get a custom label for use when submitting jobs with `phylum analyze`.
+
+        Each CI platform/environment has unique ways of referencing events, PRs, branches, etc.
+        However, each implementation is expected to at least:
+          * Start the label with the `self.ci_platform_name`
+          * Replace all runs of whitespace characters with a single `-` character
+        """
+        raise NotImplementedError()
+
+    @property
+    def lockfile_hash_object(self) -> str:
+        """Get the lockfile hash object of the first changed lockfile and return it.
+
+        Since there can be many changed lockfiles, find and use only the hash object of the first changed lockfile.
+        Since it is possible that no lockfile has changed (e.g., when forcing analysis), default to first lockfile.
+        When found, only the first seven characters of the hash object will be returned, which is a git "short SHA-1".
+        Reference: https://git-scm.com/book/en/v2/Git-Tools-Revision-Selection
+        """
+        if not self.lockfiles:
+            return "unknown"
+        first_changed_lockfile = self.lockfiles[0]
+        for lockfile in self.lockfiles:
+            if lockfile.is_lockfile_changed:
+                first_changed_lockfile = lockfile
+                break
+        lockfile_hash_object = git_hash_object(first_changed_lockfile.path)
+        return lockfile_hash_object[:7]
+
+    @cached_property
+    @abstractmethod
+    def common_ancestor_commit(self) -> Optional[str]:
+        """Find the common ancestor commit.
+
+        When found, it should be returned as a string of the SHA1 sum representing the commit.
+        When it can't be found (or there is an error), `None` should be returned.
+        """
+        raise NotImplementedError()
+
+    @property
+    @abstractmethod
+    def is_any_lockfile_changed(self) -> bool:
+        """Get the lockfiles' collective modification status.
+
+        Implementations should return `True` if any lockfile has changed and `False` otherwise.
+        """
+        raise NotImplementedError()
+
+    def update_lockfiles_change_status(self, commit: str, err_msg: Optional[str] = None) -> None:
+        """Update each lockfile's change status.
+
+        The input `commit` is the one to use in a `git diff` command to view the changes relative to the working tree.
+        The input `err_msg` is what will be printed when `git diff` fails. This is usually due to not having enough
+        branch history...which can happen with shallow clones.
+        """
+        for lockfile in self.lockfiles:
+            print(f" [*] Checking `{lockfile!r}` for changes ...")
+            # `--exit-code` will make git exit with 1 if there were differences while 0 means no differences.
+            # Any other exit code is an error and a reason to re-raise.
+            cmd = ["git", "diff", "--exit-code", "--quiet", commit, "--", str(lockfile.path)]
+            ret = subprocess.run(cmd, check=False)
+            if ret.returncode == 0:
+                print(f" [-] The lockfile `{lockfile!r}` has not changed")
+                lockfile.is_lockfile_changed = False
+            elif ret.returncode == 1:
+                print(f" [-] The lockfile `{lockfile!r}` has changed")
+                lockfile.is_lockfile_changed = True
+            else:
+                if err_msg:
+                    print(textwrap.dedent(err_msg))
+                ret.check_returncode()
+
+    @abstractmethod
+    def _check_prerequisites(self) -> None:
+        """Ensure the necessary pre-requisites are met and bail when they aren't.
+
+        The current pre-requisites for *all* CI environments/platforms are:
+          * A Phylum CLI version with ability to specify multiple lockfiles
+          * Have `git` installed and available for use on the PATH
+        """
+        print(" [+] Confirming pre-requisites ...")
+
+        if Version(self.args.version) < Version(MIN_CLI_VER_INSTALLED):
+            raise SystemExit(f" [!] The CLI version must be at least {MIN_CLI_VER_INSTALLED}")
+
+        if shutil.which("git"):
+            print(" [+] `git` binary found on the PATH")
+        else:
+            raise SystemExit(" [!] `git` is required to be installed and available on the PATH")
+
+    def ensure_project_exists(self) -> None:
+        """Ensure a Phylum project is created and in place.
+
+        A project may or may not already exist. Attempt to create the project, possibly overwriting a `.phylum_project`
+        file that already exists. Continue on without error when the specified project already exists.
+        """
+        print(f" [*] Attempting to create a Phylum project with the name: {self.phylum_project} ...")
+        cmd = [str(self.cli_path), "project", "create", self.phylum_project]
+        if self.phylum_group:
+            print(f" [-] Using Phylum group: {self.phylum_group}")
+            cmd = [str(self.cli_path), "project", "create", "--group", self.phylum_group, self.phylum_project]
+        ret = subprocess.run(cmd, check=False, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+        # The Phylum CLI will return a unique error code of 14 when a project that already
+        # exists is attempted to be created. This situation is recognized and allowed to happen
+        # since it means the project exists as expected. Any other exit code is an error.
+        if ret.returncode == 0:
+            print(f" [-] Project {self.phylum_project} created successfully.")
+            if self._project_file_already_existed:
+                print(f" [!] Overwrote previous `.phylum_project` file found at: {self._phylum_project_file}")
+        elif ret.returncode == 14:
+            print(f" [-] Project {self.phylum_project} already exists. Continuing with it ...")
+        else:
+            shell_escaped_cmd = " ".join(shlex.quote(arg) for arg in cmd)
+            print(f" [!] There was a problem creating the project with command: {shell_escaped_cmd}")
+            ret.check_returncode()
+
+    def post_output(self) -> None:
+        """Post the output of the analysis as markdown rendered for output to the terminal/logs.
+
+        Each implementation that offers analysis output in the form of comments on a pull/merge request should
+        ensure those comments are unique and not added multiple times as the review changes but no lockfile does.
+        """
+        # Pull out the project URL link, which doesn't render well
+        output = self.analysis_output.replace(self.project_url_md, "")
+
+        # Post the markdown output, rendered for terminal/log output
+        print(f" [+] Analysis output:\n{render(output)}")
+
+        # Post the project URL link separately
+        print(f"View this project in the Phylum UI: {self.project_url}\n")
+
     def analyze(self, analysis: dict) -> ReturnCode:
-        """Analyze the results gathered from passing a lockfile to `phylum analyze`."""
+        """Analyze the results gathered from passing the lockfile(s) to `phylum analyze`."""
         self._project_id = analysis.get("project", "00000000-0000-0000-0000-000000000000")
         print(f" [+] Project ID: {self.project_id}")
 
@@ -416,12 +419,17 @@ class CIBase(ABC):
             print(" [+] Considering all current dependencies ...")
             pkgs = analysis.get("packages", [])
             packages = sorted([PackageDescriptor(pkg.get("name"), pkg.get("version"), pkg.get("type")) for pkg in pkgs])
-            print(f" [+] {len(packages)} current dependencies")
+            print(f" [+] {len(packages)} unique current dependencies")
             risk_data = self.parse_risk_data(analysis, packages)
         else:
             print(" [+] Only considering newly added dependencies ...")
-            packages = self.get_new_deps()
-            print(f" [+] {len(packages)} newly added dependencies")
+            # When the `--force-analysis` flag is specified without the `--all-deps` flag, it is necessary
+            # to ensure the `is_lockfile_changed` property is set for each lockfile. Simply referencing
+            # the `is_any_lockfile_changed` property will ensure this happens.
+            if self.force_analysis and self.is_any_lockfile_changed:
+                print(" [-] Updated each lockfile's change status")
+            packages = sorted({pkg for lockfile in self.lockfiles for pkg in lockfile.new_deps})
+            print(f" [+] {len(packages)} unique newly added dependencies")
             risk_data = self.parse_risk_data(analysis, packages)
 
         returncode = ReturnCode.SUCCESS
@@ -574,3 +582,33 @@ def build_issues_list(package_result: dict, issue_flags: List[str]) -> List[Issu
                 title = pkg_issue.get("title")
                 issues.append(IssueEntry(severity, domain, title))
     return sorted(issues)
+
+
+def detect_lockfiles() -> List[Path]:
+    """Detect the lockfile(s) in use and return them.
+
+    Lockfiles that match entries in supported ignore files will be excluded.
+
+    This function makes the following assumptions:
+      * It is called from the root of a repository
+      * Ignore files only exist at the root of a repository
+      * Recursing fully into the repository is acceptable
+    """
+    cwd = Path.cwd()
+    spec = None
+    potential_ignore_files = (Path(".gitignore"), Path(".ignore"))
+
+    ignore_files = [f.resolve() for f in potential_ignore_files if f.exists()]
+    if ignore_files:
+        try:
+            lines = [line for ignore_file in ignore_files for line in ignore_file.read_text("utf-8").splitlines()]
+            spec = pathspec.PathSpec.from_lines("gitwildmatch", lines)
+        except (UnicodeDecodeError, pathspec.patterns.gitwildmatch.GitWildMatchPatternError) as err:
+            # Ignore the failure and proceed without ignore-file filtering (how ironic)
+            print(f" [!] Could not parse an ignore file: {err}")
+            print(" [-] Continuing anyway ...")
+
+    lockfiles = [path.resolve() for lockfile_glob in SUPPORTED_LOCKFILES for path in cwd.glob(f"**/{lockfile_glob}")]
+    if spec:
+        lockfiles = [path for path in lockfiles if not spec.match_file(str(path))]
+    return lockfiles
